window.Gleam.initSearch([{"doc":"chip","title":"chip","content":"# chip\n\n[![Package Version](https://img.shields.io/hexpm/v/chip)](https://hex.pm/packages/chip)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/chip/)\n\n\nChip is a gleam process registry that plays along gleam erlang/OTP `Subject` type. \n\nIt lets us group subjects of the same type so that we can later reference them all \nas a group, or sub-group if we decide to name them. Will also automatically delist \ndead processes.\n\n### Example\n\nLets assemble the pieces to build a simple counter actor:\n\n```gleam\npub type CounterMessage {\n  Inc\n  Current(client: process.Subject(Int))\n  Stop(client: process.Subject(Int))\n}\n\nfn handle_count(message: CounterMessage, count: Int) {\n  case message {\n    Inc -&gt; {\n      actor.continue(count + 1)\n    }\n\n    Current(client) -&gt; {\n      process.send(client, count)\n      actor.continue(count)\n    }\n\n    Stop(client) -&gt; {\n      process.send(client, count)\n      actor.Stop(process.Normal)\n    }\n  }\n}\n```\n\nWe start our registry and create new instances of a counter:\n```gleam\nimport gleam/erlang/process\nimport chip\n\npub fn main() {\n  let assert Ok(registry) = chip.start()\n\n  chip.register(registry, fn() { \n    actor.start(1, handle_count)\n  })\n  \n  chip.register(registry, fn() { \n    actor.start(2, handle_count)\n  })\n  \n  chip.register(registry, fn() { \n    actor.start(3, handle_count)\n  })\n}\n```\n\nThen retrieve all registered subjects so we can send messages:\n```gleam\nchip.all(registry) \n|&gt; list.each(process.send(_, Inc))\n\nlet assert [2, 3, 4] =  \n  chip.all(registry)\n  |&gt; list.map(process.call(_, Current(_), 10))\n  |&gt; list.sort(int.compare)\n```\n\nIt is also possible to register a subject under a named subgroup: \n```gleam\nimport gleam/erlang/process\nimport chip\n\ntype Group {\n  GroupA \n  GroupB \n}\n\npub fn main() {\n  let assert Ok(registry) = chip.start()\n\n  chip.register_as(registry, GroupA, fn() { \n    actor.start(1, handle_count)\n  })\n  \n  chip.register(registry, GroupB, fn() { \n    actor.start(2, handle_count)\n  })\n  \n  chip.register(registry, GroupA, fn() { \n    actor.start(3, handle_count)\n  })\n}\n```\n\nThen lookup for specific names under the group:\n```gleam\nlet assert [1, 3] = \n  chip.lookup(registry, GroupA) \n  |&gt; list.map(process.call(_, Current(_), 10))\n  |&gt; list.sort(int.compare)\n```\n\nFeature-wise this is near beign complete. Still planning to integrate: \n\n* A `via` helper to initialize processes explicitly, currently `register` and `register_as`).\n* A `register` helper to implicitly initialize subjects.\n* Pending to document guides and use-cases.\n\n## Installation\n\n```sh\ngleam add chip\n```","url":"index.html"},{"doc":"chip","title":"Message","content":"pub opaque type Message(name, message)\n\n\nSynonyms:\nchip.Message\nchip Message","url":"chip.html#Message"},{"doc":"chip","title":"all","content":"pub fn all(registry: Subject(Message(a, b))) -&gt; List(Subject(b))\n Returns all registered `Subject`s.\n \n ### Example\n \n \n &gt; chip.all(registry) \n [subject_a, subject_b, subject_c]\n \nSynonyms:\nchip.all\nchip all","url":"chip.html#all"},{"doc":"chip","title":"deregister","content":"pub fn deregister(\n  registry: Subject(Message(a, b)),\n  name: a,\n) -&gt; Nil\n Manually deregister a named group of `Subject`s.\n \n ## Example\n \n \n &gt; chip.deregister(registry, &quot;MySubjects&quot;)\n Nil\n \nSynonyms:\nchip.deregister\nchip deregister","url":"chip.html#deregister"},{"doc":"chip","title":"lookup","content":"pub fn lookup(\n  registry: Subject(Message(a, b)),\n  name: a,\n) -&gt; List(Subject(b))\n Looks up named subgroup of `Subject`s.\n \n ### Example\n \n \n &gt; chip.lookup(registry, &quot;MySubjects&quot;) \n [subject_a, subject_c]\n \nSynonyms:\nchip.lookup\nchip lookup","url":"chip.html#lookup"},{"doc":"chip","title":"named","content":"pub fn named(\n  registry: Subject(Message(a, b)),\n) -&gt; List(#(a, Subject(b)))\n Returns all named `Subject&#39;s` indexed by its key.\n\n ### Example\n \n \n &gt; chip.named(registry) \n [#(GroupA, subject_a), #(GroupA, subject_b), #(GroupC, subject_c)]\nSynonyms:\nchip.named\nchip named","url":"chip.html#named"},{"doc":"chip","title":"register","content":"pub fn register(\n  registry: Subject(Message(a, b)),\n  start: fn() -&gt; Result(Subject(b), StartError),\n) -&gt; Result(Subject(b), StartError)\n Manually registers a `Subject`. \n \n ## Example\n \n \n &gt; chip.register(registry, fn() { start_my_subject() })\n Ok(registered_subject)\n \nSynonyms:\nchip.register\nchip register","url":"chip.html#register"},{"doc":"chip","title":"register_as","content":"pub fn register_as(\n  registry: Subject(Message(a, b)),\n  name: a,\n  start: fn() -&gt; Result(Subject(b), StartError),\n) -&gt; Result(Subject(b), StartError)\n Manually registers a `Subject` under a named group. \n \n ## Example\n \n \n &gt; chip.register(registry, &quot;MySubjects&quot;, fn() { start_my_subject() })\n Ok(registered_subject)\n \nSynonyms:\nchip.register_as\nchip register_as","url":"chip.html#register_as"},{"doc":"chip","title":"start","content":"pub fn start() -&gt; Result(Subject(Message(a, b)), StartError)\n Starts the registry.\n \n ## Example\n \n \n &gt; chip.start()\n Ok(registry)\n \nSynonyms:\nchip.start\nchip start","url":"chip.html#start"},{"doc":"chip","title":"stop","content":"pub fn stop(registry: Subject(Message(a, b))) -&gt; ExitReason\n Stops the registry, all grouped `Subject`s will be gone.\n \n ## Example\n \n \n &gt; chip.stop(registry)\n Normal\n \nSynonyms:\nchip.stop\nchip stop","url":"chip.html#stop"},{"doc":"chip","title":"chip","content":"Chip is a gleam process registry that plays along gleam erlang/OTP `Subject` type. \n\nIt lets us group subjects of the same type so that we can later reference them all \nas a group, or sub-group if we decide to name them. Will also automatically delist \ndead processes.","url":"chip.html"}]);